/*
Copyright © 2025 filifa

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
package cmd

import (
	"fmt"
	"os"

	"scm.dairydemon.net/filifa/dptdist/cmd/internal/markov"

	"github.com/spf13/cobra"
	"gonum.org/v1/gonum/graph/encoding/dot"
	"gonum.org/v1/gonum/mat"
)

var file string

// rootCmd represents the base command when called without any subcommands
var rootCmd = &cobra.Command{
	Use:   "dptdist",
	Short: "A brief description of your application",
	Long: `A longer description that spans multiple lines and likely contains
examples and usage of using your application. For example:

Cobra is a CLI library for Go that empowers applications.
This application is a tool to generate the needed files
to quickly create a Cobra application.`,
	// Uncomment the following line if your bare application
	// has an action associated with it:
	Run: parse,
}

func parse(cmd *cobra.Command, args []string) {
	data, err := os.ReadFile(file)
	if err != nil {
		panic(err)
	}

	graph := markov.NewAbsorbingMarkovChain()
	err = dot.Unmarshal(data, graph)
	if err != nil {
		panic(err)
	}

	if !graph.IsValid() {
		panic("not an absorbing Markov chain!")
	}

	a := graph.AdjacencyMatrix()
	out := mat.Formatted(a)
	fmt.Printf("%v\n", out)
}

// Execute adds all child commands to the root command and sets flags appropriately.
// This is called by main.main(). It only needs to happen once to the rootCmd.
func Execute() {
	err := rootCmd.Execute()
	if err != nil {
		os.Exit(1)
	}
}

func init() {
	// Here you will define your flags and configuration settings.
	// Cobra supports persistent flags, which, if defined here,
	// will be global for your application.

	// rootCmd.PersistentFlags().StringVar(&cfgFile, "config", "", "config file (default is $HOME/.dptdist.yaml)")

	// Cobra also supports local flags, which will only run
	// when this action is called directly.
	rootCmd.Flags().BoolP("toggle", "t", false, "Help message for toggle")

	rootCmd.Flags().StringVarP(&file, "file", "f", "", "dot file with absorbing Markov chain")
	rootCmd.MarkFlagRequired("file")
}
